/* -*- linux-c -*-
 *
 * SW_64 dwarf unwinder header file
 * Copyright (C) 2011 Red Hat, Inc.
 *
 * This file is part of systemtap, and is free software.  You can
 * redistribute it and/or modify it under the terms of the GNU General
 * Public License (GPL); either version 2, or (at your option) any
 * later version.
 */
#ifndef _STP_SW_64_UNWIND_H
#define _STP_SW_64_UNWIND_H

#include <linux/sched.h>
#include <asm/ptrace.h>

#define _stp_get_unaligned(ptr) (*(ptr))
#ifdef OS510

#define UNW_PC(frame)        (frame)->regs.pc
#define UNW_SP(frame)        (frame)->regs.regs[30]
#define UNW_GP(frame)        (frame)->regs.gp
#define STACK_LIMIT(ptr)     (((ptr) - 1) & ~(THREAD_SIZE - 1))

#define UNW_REGISTER_INFO \
	PTREGS_INFO(regs[0]), \
	PTREGS_INFO(regs[1]), \
	PTREGS_INFO(regs[2]), \
	PTREGS_INFO(regs[3]), \
	PTREGS_INFO(regs[4]), \
	PTREGS_INFO(regs[5]), \
	PTREGS_INFO(regs[6]), \
	PTREGS_INFO(regs[7]), \
	PTREGS_INFO(regs[8]), \
	PTREGS_INFO(regs[9]), \
	PTREGS_INFO(regs[10]), \
	PTREGS_INFO(regs[11]), \
	PTREGS_INFO(regs[12]), \
	PTREGS_INFO(regs[13]), \
	PTREGS_INFO(regs[14]), \
	PTREGS_INFO(regs[15]), \
	PTREGS_INFO(regs[16]), \
	PTREGS_INFO(regs[17]), \
	PTREGS_INFO(regs[18]), \
	PTREGS_INFO(regs[19]), \
	PTREGS_INFO(regs[20]), \
	PTREGS_INFO(regs[21]), \
	PTREGS_INFO(regs[22]), \
	PTREGS_INFO(regs[23]), \
	PTREGS_INFO(regs[24]), \
	PTREGS_INFO(regs[25]), \
	PTREGS_INFO(regs[26]), \
	PTREGS_INFO(regs[27]), \
	PTREGS_INFO(regs[28]), \
	PTREGS_INFO(regs[29]), \
	PTREGS_INFO(regs[30]), \
	PTREGS_INFO(regs[31]), \
	PTREGS_INFO(pc)

#define DWARF_REG_MAP(r) \
        ((r >= 0 && r <= 32) ? r /* regs[0-30] + sp */	\
	: 9999)

#define UNW_PC_IDX 32
#define UNW_SP_IDX 30
#define UNW_GP_IDX 29

#else

#define UNW_PC(frame)        (frame)->regs.pc
#define UNW_SP(frame)        (frame)->regs.sp
#define UNW_GP(frame)        (frame)->regs.gp
#define STACK_LIMIT(ptr)     (((ptr) - 1) & ~(THREAD_SIZE - 1))

#define UNW_REGISTER_INFO \
	PTREGS_INFO(r0), \
	PTREGS_INFO(r1), \
	PTREGS_INFO(r2), \
	PTREGS_INFO(r3), \
	PTREGS_INFO(r4), \
	PTREGS_INFO(r5), \
	PTREGS_INFO(r6), \
	PTREGS_INFO(r7), \
	PTREGS_INFO(r8), \
	PTREGS_INFO(r9), \
	PTREGS_INFO(r10), \
	PTREGS_INFO(r11), \
	PTREGS_INFO(r12), \
	PTREGS_INFO(r13), \
	PTREGS_INFO(r14), \
	PTREGS_INFO(r15), \
	PTREGS_INFO(r16), \
	PTREGS_INFO(r17), \
	PTREGS_INFO(r18), \
	PTREGS_INFO(r19), \
	PTREGS_INFO(r20), \
	PTREGS_INFO(r21), \
	PTREGS_INFO(r22), \
	PTREGS_INFO(r23), \
	PTREGS_INFO(r24), \
	PTREGS_INFO(r25), \
	PTREGS_INFO(r26), \
	PTREGS_INFO(r27), \
	PTREGS_INFO(r28), \
	PTREGS_INFO(gp), \
	PTREGS_INFO(sp), \
	PTREGS_INFO(pc), \

#define DWARF_REG_MAP(r) \
        ((r >= 0 && r <= 31) ? r \
	: 9999)

#define UNW_PC_IDX 31
#define UNW_SP_IDX 30
#define UNW_GP_IDX 29
#endif

static inline void arch_unw_init_frame_info(struct unwind_frame_info *info,
                                            /*const*/ struct pt_regs *regs,
					    int sanitize)
{
	
      /*  if(regs == NULL){
	        asm("stl $r0,   0(%2) \n\t"
		    "stl $r1,   8(%2) \n\t"
		    "stl $r2,  16(%2) \n\t"
		    "stl $r3,  24(%2) \n\t"
		    "stl $r4,  32(%2) \n\t"
		    "stl $r5,  40(%2) \n\t"
		    "stl $r6,  48(%2) \n\t"
		    "stl $r7,  56(%2) \n\t"
		    "stl $r8,   64(%2) \n\t"
		    "stl $r9,   72(%2) \n\t"
		    "stl $r10,  80(%2) \n\t"
		    "stl $r11,  88(%2) \n\t"
		    "stl $r12,  96(%2) \n\t"
		    "stl $r13, 104(%2) \n\t"
		    "stl $r14, 112(%2) \n\t"
		    "stl $r15, 120(%2) \n\t"
		    "stl $r16, 128(%2) \n\t"
                    "stl $r17, 136(%2) \n\t"
                    "stl $r18, 144(%2) \n\t"
		    "stl $r19, 152(%2) \n\t"
                    "stl $r20, 160(%2) \n\t"
                    "stl $r21, 168(%2) \n\t"
		    "stl $r22, 176(%2) \n\t"
                    "stl $r23, 184(%2) \n\t"
		    "stl $r24, 192(%2) \n\t"
		    "stl $r25, 200(%2) \n\t"
		    "stl $r26, 208(%2) \n\t"
	            "stl $r27, 216(%2) \n\t"
		    "stl $r28, 224(%2) \n\t"
		    "stl gp, 232(%2) \n\t"
		    "stl sp, 240(%2) \n\t"
		    "stl pc, 248(%2) \n\t"
		    : "=m" (info->regs),
		      "=r" (info->regs.pc)
		    : "r" (&info->regs)
		    );
	        return;
        }*/
	if (&info->regs == regs) { /* happens when unwinding kernel->user */
		info->call_frame = 1;
		return;
	}

	memset(info, 0, sizeof(*info));
	info->regs = *regs;
}

#endif /* _STP_SW_64_UNWIND_H */
